# -*- coding: utf-8 -*-
# BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
# 
# This module is under the UIUC open-source license. See 
# github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
# for license details.
"""
"""
import os
import numpy as np
import biosteam as bst
import thermosteam as tmo
from biosteam import units
from biosteam.plots import plot_contour_2d, MetricBar, CABBI_green_colormap
from matplotlib import pyplot as plt
from warnings import filterwarnings
import biorefineries.fattyalcohols as fa
from biosteam import main_flowsheet as F
from thermosteam.reaction import (Reaction as Rxn,
                                  ParallelReaction as ParallelRxn,
)

__all__ = ('create_system',)

def create_system(ID='fattyalcohol_sys'):
    chemicals = fa.create_chemicals()
    bst.settings.set_thermo(chemicals)
    fattyalcohol_production_sys = fa.create_fattyalcohol_production_sys()
    # TODO: Add separation system
    return fattyalcohol_production_sys


# -*- coding: utf-8 -*-
# BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
# 
# This module is under the UIUC open-source license. See 
# github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
# for license details.
"""
"""

__all__ = ('create_chemicals',)

def create_chemicals():
    from biorefineries.lipidcane import chemicals
    return chemicals.subgroup(
        ['Ash', 'Cellulose', 'Hemicellulose', 'Lignin',
         'Glucose', 'Sucrose', 'Solids', 'Water', 'Ethanol',
         'Octane', 'DryYeast', 'H3PO4', 'P4O10', 'CaO', 'Flocculant', 'CO2',
         'O2', 'CH4']
    )


# -*- coding: utf-8 -*-
"""
Created on Mon Sep 16 00:02:57 2019

@author: yoelr
"""
import pandas as pd
from thermosteam.utils import colors
from biosteam.plots import plot_spearman
from biorefineries.cornstover.model import cornstover_model

# %%  Plot Spearman correlations

# Replacement parameter labels
replacement_labels = {
    'Stream-Ethanol price': 'Ethanol price',
    'TEA operating days': 'Operating days',
    'Stream-cornstover price': 'Cornstover price',
    'Fermentation-R301 efficiency': 'Fermentation efficiency',
    'Stream-cellulase price': 'Cellulase price',
    'Stream-cornstover flow rate': 'Cornstover flow rate',
    'TEA income tax': 'Income tax',
    'Saccharification and co fermentation-R301 saccharification conversion': 'Saccharification conversion',
    'Saccharification and co fermentation-R301 ethanol conversion': 'Fermentation ethanol conversion',
    'Boiler turbogenerator-BT boiler efficiency': 'Boiler efficiency',
    'Boiler turbogenerator boiler base cost': 'Boiler baseline cost',
    'Boiler turbogenerator turbogenerator base cost': 'Turbogenerator baseline cost',
    'Pretreatment reactor system base cost': 'Pretreatment reactor baseline cost',
    'Power utility price': 'Electricity price',
    'Cooling tower base cost': 'Cooling tower baseline cost',
    'Waste water system cost waste water system base cost': 'Wastewater treatment baseline cost',
    'Waste water system cost waste water system exponent': 'Wastewater treatment exponent'}


def var_to_label(var):
    """Replace label text for graph."""
    description = var.describe()
    name, distribution = description.split(' (')
    lb, mid, ub = eval('(' + distribution)
    if 'efficiency' in name:
        distribution = f" ({lb:.2f}, {mid:.2f}, {ub:.2f})"
    else:
        distribution = f" ({lb:.3g}, {mid:.3g}, {ub:.3g})"
    pos = name.find(' [')
    if pos != -1:
        units = str(name[pos:])
        if units == ' [USD/kg]':
            units = ' [$\mathrm{USD} \cdot \mathrm{kg}^{-1}$]'
        elif units == ' [USD/kWhr]':
            units = ' [$\mathrm{USD} \cdot \mathrm{kWhr}^{-1}$]'
        elif units == ' [kg/hr]':
            units = ' [$\mathrm{kg} \cdot \mathrm{hr}^{-1}$]'
        name = name[:pos]
    else:
        units = ''
    if name in replacement_labels:
        name = replacement_labels[name]
    return name + units + distribution

# # Get data
rhos = pd.read_excel('results/Spearman correlation cornstover.xlsx',
                      header=[0], index_col=[0, 1]).iloc[:, 0]

# Get only important parameters
rhos = rhos[rhos.abs() > 0.055] 
labels_by_index = {i.index: var_to_label(i) for i in cornstover_model.get_parameters() 
                   if i.index in rhos.index}
labels = [labels_by_index[i] for i in rhos.index]
rhos = pd.Series(data=rhos.values, index=labels)

# # Plot and fix axis labels
fig, ax = plot_spearman(rhos, top=10, name='MESP')
# labels = [item.get_text() for item in ax.get_yticklabels()]
# new_labels = [replace_label_text(i) for i in labels]
# ax.set_yticklabels(new_labels)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Dec 21 11:05:24 2017

@author: Yoel
"""
import biosteam as bst
from biosteam import units
from ._process_settings import price

__all__ = ('create_ethanol_production_system',
           'mass2molar_ethanol_fraction')

# %% Helpful functions

def mass2molar_ethanol_fraction(x):
    """Return ethanol mol fraction in a ethanol water mixture"""
    return x/46.06844 / (x/46.06844 + (1-x)/18.01528)

# %% Pretreatment section

def create_ethanol_production_system(ID='ethanol_production_sys',
                                     sugar_solution=None):
    ### Streams ###
    
    # Fresh water
    stripping_water = bst.Stream('stripping_water', Water=5000, units='kg/hr')
    
    # Gasoline
    denaturant = bst.Stream('denaturant', Octane=230.69,
                            units='kg/hr', price=price['Gasoline'])
    
    if not sugar_solution: # Feedstock
        sugar_solution = bst.Stream('sugar_solution',
            Glucose = 3802,
            Sucrose = 4.309e+04,
            Water   = 2.59e+05,
            H3PO4   = 83.33,
            units = 'kg/hr',
            T = 372,
        )
    
    # Yeast
    yeast = bst.Stream('yeast', Water=24700, DryYeast=10300, units='kg/hr')
    
    # Ethanol product
    ethanol = bst.Stream('ethanol', price=price['Ethanol'])
    
    ### Units ###
    
    # Split sugar solution
    S301 = units.Splitter('S301',
                        split=0.265)
    
    # Concentrate sugars
    F301 = units.MultiEffectEvaporator('F301',
                                       P=(101325, 73581, 50892, 32777),
                                       V=0.95) # fraction evaporated
    F301.components['condenser'].U = 1.85
    # Note: value of steam ~ 6.86 for the following 
    # (101325, 73580.467, 50891.17, 32777.406, 19999.925, 11331.5),
    
    # Mix sugar solutions
    M301 = units.Mixer('M301')
    
    # Cool for fermentation
    H301 = units.HXutility('H301', T=295.15)
    
    # Ethanol Production
    R301 = units.Fermentation('R301', outs=('CO2', ''), tau=9, efficiency=0.90, N=4) 
    T301 = units.StorageTank('T301', tau=4, vessel_material='Carbon steel')
    T301.line = 'Beer tank'
    
    D301 = units.VentScrubber('D301', ins=(stripping_water, R301-0), 
                              outs=('vent', ''),
                              gas=('CO2',))
    
    # Separate 99% of yeast
    C301 = units.SolidsCentrifuge('C301', outs=('', 'recycle_yeast'),
                                split=(1, 0.99999, 1, 0.96, 0.01),
                                order=('Ethanol', 'Glucose', 'H3PO4', 
                                       'Water', 'DryYeast'),
                                solids=('DryYeast',))
    
    # Mix in Water
    M302 = units.Mixer('M302')
    P301 = units.Pump('P301')
    
    # Heat up before beer column
    # Exchange heat with stillage
    H302 = units.HXprocess('H302', phase0='l', phase1='l', U=1.28)
    
    # Beer column
    xbot = mass2molar_ethanol_fraction(0.00001)
    ytop = mass2molar_ethanol_fraction(0.574)
    D302 = units.BinaryDistillation('D302', P=101325,
                                y_top=ytop, x_bot=xbot, k=1.25,
                                LHK=('Ethanol', 'Water'))
    D302.tray_material = 'Stainless steel 304'
    D302.vessel_material = 'Stainless steel 304'
    D302.boiler.U = 1.85
    P302 = units.Pump('P302')
    
    # Mix ethanol Recycle (Set-up)
    M303 = units.Mixer('M303')
    
    ytop = mass2molar_ethanol_fraction(0.9061726)
    D303 = units.BinaryDistillation('D303', P=101325,
                                y_top=ytop, x_bot=xbot, k=1.25,
                                LHK=('Ethanol', 'Water'),
                                tray_material='Stainless steel 304',
                                vessel_material='Stainless steel 304',
                                is_divided=True)
    D303.boiler.U = 1.85
    P303 = units.Pump('P303')
    
    # Superheat vapor for mol sieve
    H303 = units.HXutility('H303', T=115+273.15, V=1)
    
    # Molecular sieve
    U301 = units.MolecularSieve('U301',
                                split=(2165.14/13356.04, 1280.06/1383.85),
                                order=('Ethanol', 'Water'))
    
    # Condense ethanol product
    H304 = units.HXutility('H304', 'S149', V=0, T=340.)
    T302 = units.StorageTank('T302', tau=7*24,
                             vessel_type='Floating roof',
                             vessel_material='Carbon steel')
    P304 = units.Pump('P304')
    
    # Storage for gasoline
    T303 = units.StorageTank('T303', tau=7*24,
                             vessel_type='Floating roof',
                             vessel_material='Carbon steel')
    P305 = units.Pump('P305')
    
    # Denatured ethanol product
    T304 = units.MixTank('T304', outs=ethanol)
    T304.tau = 0.10
    
    # Waste water
    M305 = units.Mixer('M305', outs='wastewater')
    
    # Yeast mixing
    T305 = units.MixTank('T305')
    T305.tau = 0.1
    yeast-T305
    
    # Multi-effect evaporator pumps
    P306 = units.Pump('P306')
    
    
    ### Ethanol system set-up ###
    
    sugar_solution-S301-1-F301-0-P306
    (S301-0, P306-0)-M301-H301
    (H301-0, yeast-T305-0)-R301-1-T301-0-C301
    (C301-0, D301-1)-M302-P301
    (P301-0, P302-0)-H302-0-D302-1-P302
    (D302-0, U301-0)-M303-0-D303-0-H303-U301
    D303-1-P303
    
    def adjust_denaturant():
        P304._run()
        pure_ethanol = P304.outs[0]
        denaturant.imol['Octane'] = 0.021*pure_ethanol.F_mass/114.232
        
    P304.specification = adjust_denaturant
    U301-1-H304-0-T302-0-P304
    denaturant-T303-P305
    (P305-0, P304-0)-T304
    (P303-0, F301-1, H302-1)-M305
    
    ### System ###
    
    return bst.System(ID, 
                [S301, 
                 F301, 
                 P306, 
                 M301, 
                 H301, 
                 T305,
                 R301,
                 T301, 
                 C301, 
                 M302, 
                 P301,
                 bst.System('beer_column_heat_integration',
                     [H302,
                      D302,
                      P302],
                     recycle=P302-0),
                 bst.System('ethanol_recycle_from_molecular_sieves',
                     [M303,
                      D303,
                      H303,
                      U301],
                     recycle=U301-0),
                 H304,
                 T302, 
                 P304,
                 T303, 
                 P305, 
                 T304, 
                 D301,
                 P303, 
                 M305])



# -*- coding: utf-8 -*-
# BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
# 
# This module is under the UIUC open-source license. See 
# github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
# for license details.
from setuptools import setup

setup(
    name='biorefineries',
    packages=['biorefineries'],
    license='MIT',
    version='2.15.13',
    description="Biorefinery models in BioSTEAM",
    long_description=open('README.rst').read(),
    author='Yoel Cortes-Pena',
    install_requires=['biosteam>=2.20.16'],
    python_requires=">=3.6",
    package_data=
        {'biorefineries': ['biorefineries/*',
                           'lipidcane/*', 
                           'lipidcane/utils/*', 
                           'cornstover/*', 
                           'cornstover/_humbird2011.xlsx',
                           'sugarcane/*',
                           'fattyalcohols/*',
                           'fattyalcohols/units',
                           'LAOs/*',
                           'LAOs/units',
                      ]},
    platforms=['Windows', 'Mac', 'Linux'],
    author_email='yoelcortes@gmail.com',
    url='https://github.com/BioSTEAMDevelopmentGroup/Bioindustrial-Park/tree/master/biorefineries/BioSTEAM 2.x.x/biorefineries',
    download_url='https://github.com/BioSTEAMDevelopmentGroup/Bioindustrial-Park/tree/master/biorefineries/BioSTEAM 2.x.x/biorefineries',
    classifiers=['Development Status :: 3 - Alpha',
                 'Environment :: Console',
                 'License :: OSI Approved :: MIT License',
                 'Programming Language :: Python :: 3.6',
                 'Programming Language :: Python :: 3.7',
                 'Topic :: Scientific/Engineering',
                 'Topic :: Scientific/Engineering :: Chemistry',
                 'Topic :: Scientific/Engineering :: Mathematics'],
    keywords='chemical process simmulation bioprocess engineering mass CABBI biorefinery biofuel bioproducts lipid-cane corn stover',
)